// ------------------------------------------------------------------------
// Space Project -- Tecgraf/PUC-Rio
// www.tecgraf.puc-rio.br
//
// $Source:  $
// $Revision:  $
// $Date:  $
//
// $Author:  $
// ------------------------------------------------------------------------

#ifdef _WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <vector>
#include <string>
#include <fstream>
#include <map>

#include <math.h>
#include <iup.h>
#include <iupgl.h>
#include <iupcontrols.h>

#include "util.h"
#include "coord.h"

using std::string;
using std::vector;
using std::ifstream;
using std::map;

#define TOL_ZERO 1.0e-08

static float cv_w, cv_h;
static bool buttonPress;
static int previous_lambda = 0;

static bool gamma = false;
static double gamma_val;

static bool press = true;
static bool emissiveMode = true;

static cRGB specsRGBColor;
static cRGB specsRGBtoHSLColor;

static cRGB specAdobeRGBColor;
static cRGB specAdobetoHSLColor;

static cRGB specAppleRGBColor;
static cRGB specAppletoHSLColor;

static cRGB positiveError = 0.0;
static cRGB negativeError = 0.0;

static cRGB* currentColor;

static vector<float> points(401);

static map<string, vector<float> > iluminant;
static vector<cCoord> ref_pts;
static vector<cRGB> color_pts;

static string exec_path;
static string currentRef;

static Ihandle* canvas;
static Ihandle* sRGBCanvas;
static Ihandle* adobeRGBCanvas;
static Ihandle* appleRGBCanvas;
static Ihandle* positiveErrorCanvas;
static Ihandle* negativeErrorCanvas;

//-----------------------------------------------------------
//-----------------------------------------------------------
void SetExecPath(char* s)
{
 exec_path = DropFilename((string)s);
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void SetCurrentMainCanvas(Ihandle* cv)
{
 canvas = cv;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void SetsRGBCanvas(Ihandle* cv)
{
 sRGBCanvas = cv;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void SetAdobeRGBCanvas(Ihandle* cv)
{
 adobeRGBCanvas = cv;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void SetAppleRGBCanvas(Ihandle* cv)
{
 appleRGBCanvas = cv;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void SetnegErrorCanvas(Ihandle* cv)
{
 negativeErrorCanvas = cv;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void SetposErrorCanvas(Ihandle* cv)
{
 positiveErrorCanvas = cv;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static void XYZtosRGB(cRGB* color, cCoord p)
{
 color->r =  (float)3.2406*p.x - (float)1.5372*p.y - (float)0.4986*p.z;
 color->g = (float)-0.9689*p.x + (float)1.8758*p.y + (float)0.0415*p.z;
 color->b =  (float)0.0557*p.x - (float)0.2040*p.y + (float)1.0570*p.z;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static void XYZtoAdobeRGB(cRGB* color, cCoord p)
{
 color->r = (float)2.04148*p.x   - (float)0.564977*p.y - (float)0.344713*p.z;
 color->g = (float)-0.969258*p.x + (float)1.87599*p.y  + (float)0.0415557*p.z;
 color->b = (float)0.0134455*p.x - (float)0.118373*p.y + (float)1.01527*p.z;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static void XYZtoAppleRGB(cRGB* color, cCoord p)
{
 color->r = (float)2.95176*p.x   - (float)1.28951*p.y  - (float)0.47388*p.z;
 color->g = (float)-1.0851*p.x   + (float)1.99084*p.y  + (float)0.0372023*p.z;
 color->b = (float)0.0854804*p.x - (float)0.269456*p.y + (float)1.09113*p.z;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void GammaCorrection(cRGB* color)
{
  //Gamma Correction 2.5
 if(color->r > 0.0031308)
  color->r = (float)(1.055*(pow((float)color->r, (float)(1.0/gamma_val)))) - 0.055;
 else
  color->r *= (float)12.92;

 if(color->g > 0.0031308)
  color->g = (float)(1.055*(pow((float)color->g, (float)(1.0/gamma_val)))) - 0.055;
 else
  color->g *= (float)12.92;

 if(color->b > 0.0031308)
  color->b = (float)(1.055*(pow((float)color->b, (float)(1.0/gamma_val)))) - 0.055;
 else
  color->b *= (float)12.92;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void toHSL(cRGB *color)
{
 float var_min = Min(color->r, Min(color->g, color->b));
 float var_max = Max(color->r, Max(color->g, color->b));
 float del_max = var_max - var_min;

 float L = (var_max + var_min)/2;
 float H = 0.0;
 float S = 0.0;

 float del_r, del_g, del_b;
 if(del_max > TOL_ZERO)
 {
  if(L < 0.5)
   S = del_max / ( var_max + var_min );
  else
   S = del_max / ( 2 - var_max - var_min );

  del_r = (((var_max - color->r)/6) + (del_max/2))/del_max;
  del_g = (((var_max - color->g)/6) + (del_max/2))/del_max;
  del_b = (((var_max - color->b)/6) + (del_max/2))/del_max;

   if(fabs((color->r-var_max)) < TOL_ZERO)
    H = del_b - del_g;
   else if(fabs((color->g-var_max)) < TOL_ZERO)
    H = (1/3) + del_r - del_b;
   else if(fabs((color->b-var_max)) < TOL_ZERO)
    H = (2/3) + del_g - del_r;

   if(H < 0)
    H += 1;
   if(H > 1)
    H -= 1;
 }
 color->r = H;
 color->g = S;
 color->b = L;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void loadColorMatchFunc(std::ifstream& file)
{
 cCoord p;
 if(!ref_pts.empty())
  ref_pts.clear();
 while(!file.eof())
 {
  file >> p.x;
  file >> p.y;
  file >> p.z;
  ref_pts.push_back(p);
 }
}

//-----------------------------------------------------------
//-----------------------------------------------------------
bool loadIluminant(std::ifstream& file, string filename)
{
 float v;
 vector<float> vec;

 while(!file.eof())
 {
  file >> v;
  vec.push_back(v);
 }

 char att[64];
 if(vec.size() < 401)
 {
  sprintf(att, "Reference White %s not loaded.", (char*)filename.c_str());
  IupMessage("Spectrum Lab Error", att);
  return false;
 }
 else
 {
  iluminant[filename] = vec;
  return true;
 }
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void RedrawColorBrowser(void)
{
 char att[100];
 sprintf(att, "%.f %.f %.f", currentColor->r*255, currentColor->g*255, currentColor->b*255);
 IupStoreAttribute(IupGetHandle("color_display"), "RGB", att);
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void RedrawAllCanvas(void)
{
 IupRefresh(canvas);
 IupRefresh(sRGBCanvas);
 IupRefresh(adobeRGBCanvas);
 IupRefresh(appleRGBCanvas);
 IupRefresh(positiveErrorCanvas);
 IupRefresh(negativeErrorCanvas);
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static void clamp(cRGB* color)
{
 float min, max, r;

 min = Min(color->r, Min(color->g, color->b));
 if(min < TOL_ZERO)
 {
  r = -min;
  *color += r;
  negativeError = r;
 }

 max = Max(color->r, Max(color->g, color->b));
 if(max > 1.0)
 {
  *color /= max;
  positiveError = max-1.0;
 }
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static void specColors(void)
{
 cRGB color;
 color_pts.clear();
 int i, size = (int)ref_pts.size();
 for(i = 0; i <= size; i++)
 {
  XYZtosRGB(&color, ref_pts[i]);

  clamp(&color);

  if(gamma)
   GammaCorrection(&color);

  
  
  color_pts.push_back(color);
 }
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static void calcColor(void)
{
 int i;
 float white = 0.0;
 cCoord p;
 p.x = 0.0;
 p.y = 0.0;
 p.z = 0.0;

 int size = (int)points.size();

 if(emissiveMode)
 {
  for(i = 0; i < size; i++)
  {
   p += ref_pts[i]*points[i];
   white += ref_pts[i].y;
  }
 }
 else
 {
  vector<float> vec = iluminant[currentRef];
  for(i = 0; i < size; i++)
  {
   p += ref_pts[i]*points[i]*vec[i];
   white += vec[i]*ref_pts[i].y;
  }
 }

 p /= white;

 XYZtosRGB(&specsRGBColor, p);
 XYZtoAdobeRGB(&specAdobeRGBColor, p);
 XYZtoAppleRGB(&specAppleRGBColor, p);
 clamp(&specsRGBColor);
 clamp(&specAdobeRGBColor);
 clamp(&specAppleRGBColor);
 if(gamma)
 {
   GammaCorrection(&specsRGBColor);
   GammaCorrection(&specAdobeRGBColor);
   GammaCorrection(&specAppleRGBColor);
 }

 specsRGBtoHSLColor = specsRGBColor;
 specAdobetoHSLColor = specAdobeRGBColor;
 specAppletoHSLColor = specAppleRGBColor;

 toHSL(&specsRGBtoHSLColor);
 toHSL(&specAdobetoHSLColor);
 toHSL(&specAppletoHSLColor);
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static int cbExit(void)
{
 return IUP_CLOSE;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
int cbEmissiveToggle(Ihandle*, int v)
{
 if(v)
 {
  emissiveMode = true;
  calcColor();
  RedrawColorBrowser();
  RedrawAllCanvas();
 }
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
int cbReflectiveToggle(Ihandle*, int v)
{
 if(v)
 {
  emissiveMode = false;
  calcColor();
  RedrawColorBrowser();
  RedrawAllCanvas();
 }
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
int cbReferenceWhite(Ihandle*, char* t, int i, char v)
{
 if(v)
 {
  currentRef = t;
  calcColor();
  RedrawColorBrowser();
  RedrawAllCanvas();
 }
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
int cbGammaToggle(Ihandle*, int v)
{
 if(v)
 {
  gamma = true;
  IupSetAttribute(IupGetHandle("gamma_val"), "ACTIVE", "YES");
 }
 else
 {
  gamma = false;
  IupSetAttribute(IupGetHandle("gamma_val"), "ACTIVE", "NO");
 }
  
 specColors();
 calcColor();
 RedrawColorBrowser();
 RedrawAllCanvas();

 
 
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
int cbGammaVal(Ihandle* self, double val)
{
 gamma_val = val;

 specColors();
 calcColor();
 RedrawColorBrowser();
 RedrawAllCanvas();

 char att[64];
 sprintf(att, "Gamma = %.1f", gamma_val);
 IupSetAttribute(self, "TIP", att);
 
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
int cbsRGBToggle(Ihandle*, int v)
{
 if(v)
 {
  currentColor = &specsRGBColor;
  RedrawColorBrowser();
 }
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
char* cbGetsRGB(Ihandle*, int r, int c)
{
 static char att[64];
 if(c == 1)
 {
  switch(r)
  {
  case 1:
   return "R";
  case 2:
   return "G";
  case 3:
   return "B";
  }
 }
 else if(c == 2)
 {
  switch(r)
  {
  case 1:
   sprintf(att, "%.0f", specsRGBColor.r*255.0);
   return att;
  case 2:
   sprintf(att, "%.0f", specsRGBColor.g*255.0);
   return att;
  case 3:
   sprintf(att, "%.0f", specsRGBColor.b*255.0);
   return att;
  }
 }
 else if(c == 3)
 {
  switch(r)
  {
  case 1:
   return "H";
  case 2:
   return "S";
  case 3:
   return "L";
  }
 }
 else if(c == 4)
 {
  switch(r)
  {
  case 1:
   sprintf(att, "%.0f", specsRGBtoHSLColor.r*240.0);
   return att;
  case 2:
   sprintf(att, "%.0f", specsRGBtoHSLColor.g*240.0);
   return att;
  case 3:
   sprintf(att, "%.0f", specsRGBtoHSLColor.b*240.0);
   return att;
  }
 }
 return "";
}

//-----------------------------------------------------------
//-----------------------------------------------------------
int cbEditsRGB(Ihandle*, int, int, int)
{
 return IUP_IGNORE;
}


//-----------------------------------------------------------
//-----------------------------------------------------------
int cbAdobeRGBToggle(Ihandle*, int v)
{
 if(v)
 {
  currentColor = &specAdobeRGBColor;
  RedrawColorBrowser();
 }
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
char* cbGetAdobeRGB(Ihandle*, int r, int c)
{
 static char att[64];
 if(c == 1)
 {
  switch(r)
  {
  case 1:
   return "R";
  case 2:
   return "G";
  case 3:
   return "B";
  }
 }
 else if(c == 2)
 {
  switch(r)
  {
  case 1:
   sprintf(att, "%.0f", specAdobeRGBColor.r*255.0);
   return att;
  case 2:
   sprintf(att, "%.0f", specAdobeRGBColor.g*255.0);
   return att;
  case 3:
   sprintf(att, "%.0f", specAdobeRGBColor.b*255.0);
   return att;
  }
 }
 else if(c == 3)
 {
  switch(r)
  {
  case 1:
   return "H";
  case 2:
   return "S";
  case 3:
   return "L";
  }
 }
 else if(c == 4)
 {
  switch(r)
  {
  case 1:
   sprintf(att, "%.0f", specAdobetoHSLColor.r*255.0);
   return att;
  case 2:
   sprintf(att, "%.0f", specAdobetoHSLColor.g*255.0);
   return att;
  case 3:
   sprintf(att, "%.0f", specAdobetoHSLColor.b*255.0);
   return att;
  }
 }
 return "";
}

//-----------------------------------------------------------
//-----------------------------------------------------------
int cbEditAdobeRGB(Ihandle*, int, int, int)
{
 return IUP_IGNORE;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
int cbAppleRGBToggle(Ihandle*, int v)
{
 if(v)
 {
  currentColor = &specAppleRGBColor;
  RedrawColorBrowser();
 }
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
char* cbGetAppleRGB(Ihandle*, int r, int c)
{
 static char att[64];
 if(c == 1)
 {
  switch(r)
  {
  case 1:
   return "R";
  case 2:
   return "G";
  case 3:
   return "B";
  }
 }
 else if(c == 2)
 {
  switch(r)
  {
  case 1:
   sprintf(att, "%.0f", specAppleRGBColor.r*255.0);
   return att;
  case 2:
   sprintf(att, "%.0f", specAppleRGBColor.g*255.0);
   return att;
  case 3:
   sprintf(att, "%.0f", specAppleRGBColor.b*255.0);
   return att;
  }
 }
 else if(c == 3)
 {
  switch(r)
  {
  case 1:
   return "H";
  case 2:
   return "S";
  case 3:
   return "L";
  }
 }
 else if(c == 4)
 {
  switch(r)
  {
  case 1:
   sprintf(att, "%.0f", specAppletoHSLColor.r*255.0);
   return att;
  case 2:
   sprintf(att, "%.0f", specAppletoHSLColor.g*255.0);
   return att;
  case 3:
   sprintf(att, "%.0f", specAppletoHSLColor.b*255.0);
   return att;
  }
 }
 return "";
}

//-----------------------------------------------------------
//-----------------------------------------------------------
int cbEditAppleRGB(Ihandle*, int, int, int)
{
 return IUP_IGNORE;
}


//-----------------------------------------------------------
//-----------------------------------------------------------
void displayText(char* s, float x, float y)
{
	int i = 0 ;
	
	glColor3f((GLfloat)0.7, (GLfloat)0.7, (GLfloat)0.7);
	glRasterPos2f( (GLfloat)x, (GLfloat)y );
	
	for( i = 0 ; s[i] ; i++ )
	  glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, s[i]);

}

//-----------------------------------------------------------
//-----------------------------------------------------------
void drawLimits()
{
 glBegin(GL_QUADS);				
		glColor3f((GLfloat)0.0, (GLfloat)0.0, (GLfloat)0.0);
		glVertex3f((GLfloat)-1.0, (GLfloat)1.005, (GLfloat)0.0);
		glVertex3f((GLfloat)401, (GLfloat)1.005, (GLfloat)0.0);				
		glVertex3f((GLfloat)401, (GLfloat)0.0, (GLfloat)0.0);
		glVertex3f((GLfloat)-1.0, (GLfloat)0.0, (GLfloat)0.0);		
	glEnd();

	glBegin(GL_LINE_LOOP);				
		glColor3f((GLfloat)1.0, (GLfloat)1.0, (GLfloat)1.0);
		glVertex3f((GLfloat)-1.0, (GLfloat)1.005, (GLfloat)0.0);
		glVertex3f((GLfloat)401, (GLfloat)1.005, (GLfloat)0.0);				
		glVertex3f((GLfloat)401, (GLfloat)0.0, (GLfloat)0.0);
		glVertex3f((GLfloat)-1.0, (GLfloat)0.0, (GLfloat)0.0);		
	glEnd();
	
	//marca as posies nos eixos x e y.
	glBegin(GL_LINES);		
	 glColor3f((GLfloat)1.0, (GLfloat)1.0, (GLfloat)1.0);	    
		
		//posicoes eixo x (380 - 780)
		for(int i = 0; i < 9; i++)
  {
   if(i < 8)
   {
			 glVertex2f((GLfloat)(50*i+10), (GLfloat)0.0);
			 glVertex2f((GLfloat)(50*i+10), (GLfloat)-0.010);
    glVertex2f((GLfloat)(50*i+20),	(GLfloat)0.0);
			 glVertex2f((GLfloat)(50*i+20), (GLfloat)-0.010);
    glVertex2f((GLfloat)(50*i+30),	(GLfloat)0.0);
			 glVertex2f((GLfloat)(50*i+30), (GLfloat)-0.010);
    glVertex2f((GLfloat)(50*i+40),	(GLfloat)0.0);
			 glVertex2f((GLfloat)(50*i+40), (GLfloat)-0.010);

    glVertex2f((GLfloat)(50*i+10), (GLfloat)1.0);
			 glVertex2f((GLfloat)(50*i+10), (GLfloat)1.010);
    glVertex2f((GLfloat)(50*i+20),	(GLfloat)1.0);
			 glVertex2f((GLfloat)(50*i+20), (GLfloat)1.010);
    glVertex2f((GLfloat)(50*i+30),	(GLfloat)1.0);
			 glVertex2f((GLfloat)(50*i+30), (GLfloat)1.010);
    glVertex2f((GLfloat)(50*i+40),	(GLfloat)1.0);
			 glVertex2f((GLfloat)(50*i+40), (GLfloat)1.010);
   }
   glVertex2f((GLfloat)(50*i),	(GLfloat)0.0);
			glVertex2f((GLfloat)(50*i), (GLfloat)-0.015);

   glVertex2f((GLfloat)(50*i),	(GLfloat)1.0);
			glVertex2f((GLfloat)(50*i), (GLfloat)1.015);
		}

		//posicoes eixo y (0 - 1)
  glVertex2f((GLfloat)0.0,	(GLfloat)1.0);
		glVertex2f((GLfloat)-5.0,(GLfloat) 1.0);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.95);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.95);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.90);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.90);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.85);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.85);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.80);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.80);

		glVertex2f((GLfloat)0.0,	(GLfloat)0.75);
		glVertex2f((GLfloat)-5.0,(GLfloat) 0.75);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.70);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.70);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.65);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.65);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.60);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.60);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.55);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.55);
           
		glVertex2f((GLfloat)0.0,	(GLfloat)0.5);
		glVertex2f((GLfloat)-5.0,(GLfloat) 0.5);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.45);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.45);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.40);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.40);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.35);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.35);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.30);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.30);

		glVertex2f((GLfloat)0.0,	(GLfloat)0.25);
		glVertex2f((GLfloat)-5.0,(GLfloat) 0.25);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.20);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.20);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.15);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.15);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.10);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.10);
  glVertex2f((GLfloat)0.0,	(GLfloat)0.05);
		glVertex2f((GLfloat)-3.0,(GLfloat) 0.05);

		glVertex2f((GLfloat)0.0,	(GLfloat)0.0);
		glVertex2f((GLfloat)-5.0, (GLfloat)0.0);

  glVertex2f((GLfloat)400.0,	(GLfloat)1.0);
		glVertex2f((GLfloat)405.0, (GLfloat)1.0);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.95);
		glVertex2f((GLfloat)403.0, (GLfloat)0.95);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.90);
		glVertex2f((GLfloat)403.0, (GLfloat)0.90);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.85);
		glVertex2f((GLfloat)403.0, (GLfloat)0.85);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.80);
		glVertex2f((GLfloat)403.0, (GLfloat)0.80);

		glVertex2f((GLfloat)400.0,	(GLfloat)0.75);
		glVertex2f((GLfloat)405.0, (GLfloat)0.75);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.70);
		glVertex2f((GLfloat)403.0, (GLfloat)0.70);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.65);
		glVertex2f((GLfloat)403.0, (GLfloat)0.65);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.60);
		glVertex2f((GLfloat)403.0, (GLfloat)0.60);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.55);
		glVertex2f((GLfloat)403.0, (GLfloat)0.55);
           
		glVertex2f((GLfloat)400.0,	(GLfloat)0.5);
		glVertex2f((GLfloat)405.0, (GLfloat)0.5);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.45);
		glVertex2f((GLfloat)403.0, (GLfloat)0.45);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.40);
		glVertex2f((GLfloat)403.0, (GLfloat)0.40);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.35);
		glVertex2f((GLfloat)403.0, (GLfloat)0.35);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.30);
		glVertex2f((GLfloat)403.0, (GLfloat)0.30);

		glVertex2f((GLfloat)400.0,	(GLfloat)0.25);
		glVertex2f((GLfloat)405.0, (GLfloat)0.25);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.20);
		glVertex2f((GLfloat)403.0, (GLfloat)0.20);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.15);
		glVertex2f((GLfloat)403.0, (GLfloat)0.15);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.10);
		glVertex2f((GLfloat)403.0, (GLfloat)0.10);
  glVertex2f((GLfloat)400.0,	(GLfloat)0.05);
		glVertex2f((GLfloat)403.0, (GLfloat)0.05);

		glVertex2f((GLfloat)400.0,	(GLfloat)0.0);
		glVertex2f((GLfloat)405.0, (GLfloat)0.0);
	glEnd();
	
	//escala do eixo x
	displayText( "380", (float)-10.0, (float)-0.040 );
	displayText( "430", (float)40.0,  (float)-0.040 );
	displayText( "480", (float)90.0,  (float)-0.040 );
	displayText( "530", (float)140.0, (float)-0.040 );
	displayText( "580", (float)190.0, (float)-0.040 );
	displayText( "630", (float)240.0, (float)-0.040 );
	displayText( "680", (float)290.0, (float)-0.040 );
	displayText( "730", (float)340.0, (float)-0.040 );
	displayText( "780", (float)390.0, (float)-0.040 );	

	//escala do eixo y
	displayText( "1.0",  (float)-25.0,   (float)1.0 );
	displayText( "0.75", (float)-30.0,   (float)0.75 );
	displayText( "0.50", (float)-30.0,   (float)0.50 );
	displayText( "0.25", (float)-30.0,   (float)0.25 );
	displayText( "0.0",  (float)-25.0,   (float)0.0 );

 displayText("Wavelength (nm)", (float)135.0, (float)-0.08);		 
}


//-----------------------------------------------------------
//-----------------------------------------------------------
static void drawSpec(void)
{
 int size = (int)points.size();
 glBegin(GL_LINES);
 for(int i = 0; i < size; i++)
 {
  if(points[i] > TOL_ZERO)
  {
   glColor3f((GLfloat)color_pts[i].r, (GLfloat)color_pts[i].g, (GLfloat)color_pts[i].b);
   glVertex2f((GLfloat)i, (GLfloat)points[i]);
   glVertex2f((GLfloat)i, (GLfloat)0.005);
  }
 }
 glEnd();
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static void drawColor(cRGB color)
{
 glColor3f((GLfloat)color.r, (GLfloat)color.g, (GLfloat)color.b);
 glBegin(GL_QUADS);
  glVertex3f((GLfloat)0.0, (GLfloat)0.0, (GLfloat)0.0);
  glVertex3f((GLfloat)50.0, (GLfloat)0.0, (GLfloat)0.0);
  glVertex3f((GLfloat)50.0, (GLfloat)40.0, (GLfloat)0.0);
  glVertex3f((GLfloat)0.0, (GLfloat)40.0, (GLfloat)0.0);
 glEnd();
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static int redraw(Ihandle* self, float x, float y)
{
 IupGLMakeCurrent(self);

 /* clear buffers */
 glClearColor(0.0, 0.0, 0.0, 1.0);
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

 /* position camera */
 glMatrixMode(GL_MODELVIEW);
 glLoadIdentity();

 drawLimits();
 drawSpec();

 IupGLSwapBuffers(self);
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static int reshape(Ihandle* self, int w, int h)
{
 IupGLMakeCurrent(self);

 cv_w = (float)w;
 cv_h = (float)h;
 glViewport(0, 0, (GLint)w, (GLint)h);

 /* set projection */
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity();
 gluOrtho2D(-50.0,450,-0.1,1.1);
 glMatrixMode(GL_MODELVIEW);

 IupRefresh(self);

 return IUP_DEFAULT;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
int keyboard(Ihandle* self, int c)
{
 if(press)
 {
	 if(c == 'c')
  {
   if(gamma)
    gamma = false;
   else
    gamma = true;
  }

  specColors();
  calcColor();
  IupRefresh(self);
  IupSetAttribute(IupGetHandle("srgb_rgb_mt"), "REDRAW", "ALL");
  IupSetAttribute(IupGetHandle("adobe_rgb_mt"), "REDRAW", "ALL");
  IupSetAttribute(IupGetHandle("apple_rgb_mt"), "REDRAW", "ALL");

  press = false;
  return IUP_DEFAULT;
 }
 else
 {
  press = true;
  return IUP_IGNORE;
 }
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
int button(Ihandle* self, int but, int pressed, int x, int y, char* status)
{
 if(but == IUP_BUTTON1 && pressed == 1)
  buttonPress = true;
 else if(but == IUP_BUTTON1 && pressed == 0)
 {
  buttonPress = false;
  previous_lambda = 0;
 }
 else if(but == IUP_BUTTON3 && pressed == 1)
 {
  for(int i = 0; i < (int)points.size(); i++)
   points[i] = 0.0;

  specsRGBColor = 0.0;
  specAdobeRGBColor = 0.0;
  specAppleRGBColor = 0.0;

  calcColor();
  RedrawColorBrowser();
  RedrawAllCanvas();

  IupSetAttribute(IupGetHandle("srgb_rgb_mt"), "REDRAW", "ALL");
  IupSetAttribute(IupGetHandle("adobe_rgb_mt"), "REDRAW", "ALL");
  IupSetAttribute(IupGetHandle("apple_rgb_mt"), "REDRAW", "ALL");
 }

 return IUP_DEFAULT;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
int motion(Ihandle* self, int x, int y, char* r)
{
 IupGLMakeCurrent(self);

 if(buttonPress)
 {
  float ratio = cv_h/(float)1.2;
  float delta = (cv_h - ratio)/2;

  float energy = ((cv_h - y) - delta)/ratio;
  int lambda = x - 50;

  if(energy > TOL_ZERO && (energy - 1.0) < TOL_ZERO && lambda >= 0 && lambda <= 400)
  {
   points[lambda] = energy;
   if(previous_lambda)
   {
    // INTERPOLACAO LINEAR
    int delta_lambda, i;
    float delta_energy;

    if(lambda > previous_lambda)
    {
     delta_lambda = lambda - previous_lambda;
     delta_energy = points[lambda] - points[previous_lambda];
     for(i = previous_lambda+1; i < lambda; i++)
      float t = points[i] = points[i-1] + (delta_energy * ((i - previous_lambda) / delta_lambda));
    }
    else if(previous_lambda > lambda)
    {
     delta_lambda = previous_lambda - lambda;
     delta_energy = points[previous_lambda] - points[lambda];
     for(i = lambda+1; i < previous_lambda; i++)
      float t = points[i] = points[i-1] + (delta_energy * ((i - lambda) / delta_lambda));
    }
   }
   previous_lambda = lambda;

   calcColor();

   IupSetAttribute(IupGetHandle("srgb_rgb_mt"), "REDRAW", "ALL");
   IupSetAttribute(IupGetHandle("adobe_rgb_mt"), "REDRAW", "ALL");
   IupSetAttribute(IupGetHandle("apple_rgb_mt"), "REDRAW", "ALL");

   IupRefresh(self);

   RedrawColorBrowser();
  }
 }

 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static int sRGBRedraw(Ihandle* self, float x, float y)
{
 IupGLMakeCurrent(self);

 /* clear buffers */
 glClearColor(0.0, 0.0, 0.0, 1.0);
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

 /* position camera */
 glMatrixMode(GL_MODELVIEW);
 glLoadIdentity();

 drawColor(specsRGBColor);

 IupGLSwapBuffers(self);
 return IUP_DEFAULT;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
static int sRGBReshape(Ihandle* self, int w, int h)
{
 IupGLMakeCurrent(self);
 glViewport(0, 0, (GLint)w, (GLint)h);

 /* set projection */
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity();
 gluOrtho2D(0.0,50.0,0.0,40.0);
 glMatrixMode(GL_MODELVIEW);

 IupRefresh(self);
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static int adobeRedraw(Ihandle* self, float x, float y)
{
 IupGLMakeCurrent(self);

 /* clear buffers */
 glClearColor(0.0, 0.0, 0.0, 1.0);
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

 /* position camera */
 glMatrixMode(GL_MODELVIEW);
 glLoadIdentity();

 drawColor(specAdobeRGBColor);

 IupGLSwapBuffers(self);
 return IUP_DEFAULT;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
static int adobeReshape(Ihandle* self, int w, int h)
{
 IupGLMakeCurrent(self);
 glViewport(0, 0, (GLint)w, (GLint)h);

 /* set projection */
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity();
 gluOrtho2D(0.0,50.0,0.0,40.0);
 glMatrixMode(GL_MODELVIEW);

 IupRefresh(self);
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static int appleRedraw(Ihandle* self, float x, float y)
{
 IupGLMakeCurrent(self);

 /* clear buffers */
 glClearColor(0.0, 0.0, 0.0, 1.0);
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

 /* position camera */
 glMatrixMode(GL_MODELVIEW);
 glLoadIdentity();

 drawColor(specAppleRGBColor);

 IupGLSwapBuffers(self);
 return IUP_DEFAULT;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
static int appleReshape(Ihandle* self, int w, int h)
{
 IupGLMakeCurrent(self);
 glViewport(0, 0, (GLint)w, (GLint)h);

 /* set projection */
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity();
 gluOrtho2D(0.0,50.0,0.0,40.0);
 glMatrixMode(GL_MODELVIEW);

 IupRefresh(self);
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static int posErrorRedraw(Ihandle* self, float x, float y)
{
 IupGLMakeCurrent(self);

 /* clear buffers */
 glClearColor(0.0, 0.0, 0.0, 1.0);
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

 /* position camera */
 glMatrixMode(GL_MODELVIEW);
 glLoadIdentity();

 drawColor(positiveError);

 IupGLSwapBuffers(self);
 return IUP_DEFAULT;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
static int posErrorReshape(Ihandle* self, int w, int h)
{
 IupGLMakeCurrent(self);
 glViewport(0, 0, (GLint)w, (GLint)h);

 /* set projection */
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity();
 gluOrtho2D(0.0,50.0,0.0,40.0);
 glMatrixMode(GL_MODELVIEW);

 IupRefresh(self);
 return IUP_DEFAULT;
}

//-----------------------------------------------------------
//-----------------------------------------------------------
static int negErrorRedraw(Ihandle* self, float x, float y)
{
 IupGLMakeCurrent(self);

 /* clear buffers */
 glClearColor(0.0, 0.0, 0.0, 1.0);
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

 /* position camera */
 glMatrixMode(GL_MODELVIEW);
 glLoadIdentity();

 drawColor(negativeError);

 IupGLSwapBuffers(self);
 return IUP_DEFAULT;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
static int negErrorReshape(Ihandle* self, int w, int h)
{
 IupGLMakeCurrent(self);
 glViewport(0, 0, (GLint)w, (GLint)h);

 /* set projection */
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity();
 gluOrtho2D(0.0,50.0,0.0,40.0);
 glMatrixMode(GL_MODELVIEW);

 IupRefresh(self);
 return IUP_DEFAULT;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void setMainDialogCB(void)
{
 IupSetFunction("Exit", (Icallback)cbExit);

 IupSetFunction("emissive_cb", (Icallback)cbEmissiveToggle);
 IupSetFunction("reflective_cb", (Icallback)cbReflectiveToggle);

 IupSetFunction("ref_cb", (Icallback)cbReferenceWhite);

 IupSetFunction("gamma_cb", (Icallback)cbGammaToggle);
 IupSetFunction("ValMove_cb", (Icallback)cbGammaVal);
  
 IupSetFunction("srgb_cb", (Icallback)cbsRGBToggle);
 IupSetFunction("getsRGB_cb", (Icallback)cbGetsRGB);
 IupSetFunction("editsRGB_cb", (Icallback)cbEditsRGB);

 IupSetFunction("adobe_rgb_cb", (Icallback)cbAdobeRGBToggle);
 IupSetFunction("getAdobeRGB_cb", (Icallback)cbGetAdobeRGB);
 IupSetFunction("editAdobeRGB_cb", (Icallback)cbEditAdobeRGB);

 IupSetFunction("apple_rgb_cb", (Icallback)cbAppleRGBToggle);
 IupSetFunction("getAppleRGB_cb", (Icallback)cbGetAppleRGB);
 IupSetFunction("editAppleRGB_cb", (Icallback)cbEditAppleRGB);

 IupSetCallback(canvas, "ACTION", (Icallback)redraw);
 IupSetCallback(canvas, "RESIZE_CB", (Icallback)reshape);
 IupSetCallback(canvas, "KEYPRESS_CB", (Icallback)keyboard);
 IupSetCallback(canvas, "BUTTON_CB", (Icallback)button);
 IupSetCallback(canvas, "MOTION_CB", (Icallback)motion);

 IupSetCallback(sRGBCanvas, "ACTION", (Icallback)sRGBRedraw);
 IupSetCallback(sRGBCanvas, "RESIZE_CB", (Icallback)sRGBReshape);

 IupSetCallback(adobeRGBCanvas, "ACTION", (Icallback)adobeRedraw);
 IupSetCallback(adobeRGBCanvas, "RESIZE_CB", (Icallback)adobeReshape);

 IupSetCallback(appleRGBCanvas, "ACTION", (Icallback)appleRedraw);
 IupSetCallback(appleRGBCanvas, "RESIZE_CB", (Icallback)appleReshape);

 /*IupSetCallback(positiveErrorCanvas, "ACTION", (Icallback)posErrorRedraw);
 IupSetCallback(positiveErrorCanvas, "RESIZE_CB", (Icallback)posErrorReshape);

 IupSetCallback(negativeErrorCanvas, "ACTION", (Icallback)negErrorRedraw);
 IupSetCallback(negativeErrorCanvas, "RESIZE_CB", (Icallback)negErrorReshape);*/
 
}

//-----------------------------------------------------------
//-----------------------------------------------------------
void SetDialogToShow(void)
{
 char att[64];
 string ref;

 ref_pts.clear();
 color_pts.clear();
 iluminant.clear();

 gamma_val = 2.5;

 specsRGBColor = 0.0;
 specsRGBtoHSLColor = 0.0;

 specAdobeRGBColor = 0.0;
 specAdobetoHSLColor = 0.0;

 specAppleRGBColor = 0.0;
 specAppletoHSLColor = 0.0;

 positiveError = 0.0;
 negativeError = 0.0;

 currentColor = &specsRGBColor;
  
 int i, size = (int)points.size();
 for(i = 0; i < size; i++)
  points[i] = 0.0;

 ifstream file;

 string filename = exec_path + "database\\xyz_10deg.txt";
 file.open(filename.c_str(), std::ios::in);
 if(file)
   loadColorMatchFunc(file);
 file.close();

 file.clear();

 i = 1;
 ref = "Iluminant D65";
 filename = exec_path + "database\\ilum_d65.txt";
 file.open(filename.c_str(), std::ios::in);
 if(file)
 {
  if(loadIluminant(file, ref))
  {
   currentRef = ref;
   sprintf(att, "%d", i);
   IupStoreAttribute(IupGetHandle("spec_ref_lst"), att, (char*)ref.c_str());
   IupStoreAttribute(IupGetHandle("spec_ref_lst"), "VALUE", "1");
   i++;
  }
 }
 else
 {
  sprintf(att, "Reference White %s not loaded.", (char*)ref.c_str());
  IupMessage("Spectrum Lab Error", att);
 }
 file.close();

 if(iluminant.empty())
 {
  IupMessage("Spectrum Lab Error", "No reference white loaded. Only Emissive case available.");
  IupSetAttribute(IupGetHandle("mode_reflective_tg"), "ACTIVE", "NO");
  IupSetAttribute(IupGetHandle("spec_ref_lst"), "ACTIVE", "NO");
 }

 IupSetAttribute(IupGetHandle("gamma_val"), "ACTIVE", "NO");

 specColors();
}

